/*
  Program: ei Sudoku
  Autor: Radek Pelnek  
  Ploha ke knize "Jak to vyeit",  www.radekpelanek.cz/?jtv

  Zadn: Zkladnm clem projektu je vytvoit program, kter bude eit lohu
  Sudoku. Vstupem programu je etzec 81 znak popisujc zadn. Vstupem
  programu je pak een lohy (opt jako etzec 81 znak).

  Piloen soubory "sudoku-zadani.txt" a "sudoku-reseni.txt" obsahuj vzorov
  zadn a een.
*/

#include <iostream>
#include <string>
#include <cstdlib>
#include <iostream>

using namespace std;

//Tda reprezentujc stav plnu.
class sstate_t {
public: 
  int cell[9][9];            // hodnoty polek
  bool candidate[9][9][10];  // kandidti na jednotliv polka 
  int empty_cells;           // kolik je jet volnch pol
  bool valid;                // zda je stav konzistentn se zadnm Sudoku
  sstate_t(); 
  void copy_state(sstate_t& s);
  void read(string input);
  void print();
  void printline();
  void refine_candidates();
  void find_forced();
  void find_naked();
};

sstate_t::sstate_t() {
  for (int i=0; i<9; i++) {
    for (int j=0; j<9; j++) {
      cell[i][j] = 0;
      // na zatku jsou vechny hodnoty platn kandidti, postupn omezujeme
      for (int k=0; k<10; k++) candidate[i][j][k] = true;
    }
  }
  valid = false;
  empty_cells = 81;
}


// zkoprovn stavu
void sstate_t::copy_state(sstate_t& s) {
  for (int i=0; i<9; i++) {
    for (int j=0; j<9; j++) {
      cell[i][j] = s.cell[i][j];
      for (int k=0; k<10; k++) candidate[i][j][k] = s.candidate[i][j][k];
    }
  }
  valid = true;
  empty_cells = s.empty_cells;  
}

// naten stavu z etzce
void sstate_t::read(string input) {
  for (int i=0; i<9; i++) {
    for (int j=0; j<9; j++) {
      cell[i][j] = 0;
      for (int k=0; k<10; k++) candidate[i][j][k] = true; 
    }
  }
  empty_cells = 0;
  valid = true;
  for (int i=0; i<9; i++) {
    for (int j=0; j<9; j++) {
      if (input[j*9+i] == '.') {
	cell[i][j] = 0;
	empty_cells++;
      } else {
	cell[i][j] = input[j*9+i] - '0';
      }
    }
  }
}


// vpis stavu do mky - vhodn pro ladn
void sstate_t::print() {
  for (int j=0; j<9; j++) {
    for (int i=0; i<9; i++) {
      if (cell[i][j]) 
	cout << cell[i][j];
      else
	cout << '.';
      if (i==2 || i==5) { cout << "|"; }
    }
    cout << endl;
    if (j==2 || j==5) { cout << "-----------"<<endl; }
  }
  cout << endl;
}

// vpis stavu na jeden dek
void sstate_t::printline() {
  for (int j=0; j<9; j++) {
    for (int i=0; i<9; i++) {
      if (cell[i][j]) 
	cout << cell[i][j];
      else
	cout << '.';
    }
  }
  cout << endl;
}

// pepot mnoinu kandidt pro kad polko
void sstate_t::refine_candidates() {
  for (int j=0; j<9; j++) {
    for (int i=0; i<9; i++) {
      if (cell[i][j]) {
	int v = cell[i][j]; // hodnotu v vykrkneme z mnoiny kandidt vech bunk ve stejnm dku, sloupci a bloku
	bool oldvalue = candidate[i][j][v];
	// sloupec
	for (int k=0; k<9; k++) candidate[i][k][v] = false;
	// dek
	for (int k=0; k<9; k++) candidate[k][j][v] = false;
	// blok
	for (int k=0; k<9; k++) candidate[3*(i/3) + k%3][3*(j/3) + k/3][v] = false;
	// drobn trik, abychom v pedchozch cyklech nemuseli dlat if
	candidate[i][j][v] = oldvalue; 
      }
    }
  }
}

// hledn "forced" hodnot -- pouze jeden kandidt
void sstate_t::find_forced() {
  for (int j=0; j<9; j++) {
    for (int i=0; i<9; i++) {
      if (cell[i][j] == 0) {
	int cand = 0;
	int newval = 0;
	for (int v=1; v<=9; v++) if (candidate[i][j][v]) { cand++; newval = v; }
	if (cand==0) {
	  // pole nem dnho kandidta -- stav je neplatn
	  valid = false;
	}
	if (cand==1) {
	  // prv jeden kandidt -- doplnit
	  cell[i][j] = newval;
	  empty_cells--;
	}
      } else {
	if (! candidate[i][j][cell[i][j]]) {
	  // pole je vyplnn, ale pslun hodnota nen kandidtem -- stav je neplatn
	  valid = false;
	}
      }
    }
  }
}

// hledn "naked" hodnot -- dan hodnota me bt pouze na jedn pozici v rmci dku/sloupce/bloku
void sstate_t::find_naked() {
  // type: 0 = dek, 1 = sloupec, 2 = blok
  for (int type=0; type<3; type++) {
    for (int x=0; x<9; x++) {// hlavn index -- kolikt tvar typu type beru
      for (int v=1; v<=9; v++) { // pro vechny hodnoty projedu cel tvar
	int cand = 0;
	int candi = -1, candj = -1;
	bool isdone = false;
	for (int y=0; y<9; y++) {  // vedlej index pro prchod x-tho tvaru
	  int i, j;
	  if (type==0) { i = y; j = x; }
	  if (type==1) { i = x; j = y; }
	  if (type==2) { i = 3*(x%3) + y%3; j = 3*(x/3) + y/3; }
	  if (cell[i][j]) {
	    if (cell[i][j]==v) isdone = true;
	  } else 
	    if (candidate[i][j][v]) { cand++; candi = i; candj = j;  }	      
	}
	if (! isdone) {  // toto pole jet nen vyplnno
	  if (cand==0) {
	    // hodnota v neme bt umstna nikam v rmci tvaru
	    valid = false;
	  }
	  if (cand==1) {
	    // hodnota v me bt umstna prv na jednu pozici
	    if (! cell[candi][candj]) {
	      // pozice je przdn -- umstme slo
	      cell[candi][candj] = v;
	      empty_cells--;	     
	    } else
	      if (cell[candi][candj] != v) {
		// pozice je obsazen jinm slem -- stav je neplatn
		valid = false;
	      }
	  }
	}
      }
    }
  }
}

sstate_t solution;

// propagace podmnek
bool constraint_propagation(sstate_t& s) {
  int old_empty_cells = -1;

  // opakujeme, dokud dochz ke zmnm
  while (old_empty_cells != s.empty_cells && s.valid) {
    old_empty_cells = s.empty_cells;      
    s.refine_candidates();
    s.find_forced();
    s.find_naked();
  }
  return s.valid;
}

// prohledvn pomoc metody backtracking
bool backtrack_search(sstate_t& s) {
  
  constraint_propagation(s);
  if (! s.valid) return false;  
  if (s.empty_cells == 0) {
    // vechna pole jsou vyplnna -- mme een
    //    cout << "Reseni: " << endl;
    s.printline();
    return true;
  } else {
    // najdi voln pole
    int j = 0, i = 0;
    bool found = false;
    while (!found) {
      if (s.cell[i][j]==0) found = true;
      else {
	i++;
	if (i==9) { i = 0; j++; }
      }
    }
    // na toto pole zkus postupn doplnit vechny hodnoty od 1 do 9 a
    // rekurzivn zavolat prohledvn
    for (int v=1; v<=9; v++) {
      sstate_t succ; 
      succ.copy_state(s);
      succ.cell[i][j] = v;
      succ.empty_cells--;
      if (backtrack_search(succ)) return true; // pokud rekurzivn voln uspje, konme
    }
  }
}

int main(int argc, char** argv) {
  string in;
  sstate_t state;

  while (cin >> in) {
    state.read(in);
    //    cout << "Zadani:"<<endl<<in <<endl;
    backtrack_search(state);
  }
  
  return 0;
}

